Imports System
Imports System.Collections.Generic
Imports System.Text
Imports Infragistics.Windows.Ribbon
Imports System.Windows.Media
Imports System.Windows

Public Class ColorPickerGalleryTool
    Inherits GalleryTool
#Region "Members"

    ' The number of columns we want to have in the color picker
    Private Const GalleryColumnItems As Integer = 10

    ' The list of colors that will exist in the "Theme Colors" group.
    ' These colors are arranged as they will appear in the color picker
    ' (i.e. the first item in the third row (216,216,216) is the same color
    ' that appears in the same position in the color picker)

    Private Shared ThemeColors As Color() = New Color() { _
            Color.FromRgb(255, 255, 255), Color.FromRgb(0, 0, 0), Color.FromRgb(238, 236, 225), Color.FromRgb(31, 73, 125), Color.FromRgb(79, 129, 189), Color.FromRgb(192, 80, 77), _
            Color.FromRgb(155, 187, 89), Color.FromRgb(128, 100, 162), Color.FromRgb(75, 172, 198), Color.FromRgb(247, 150, 70), Color.FromRgb(242, 242, 242), Color.FromRgb(127, 127, 127), _
            Color.FromRgb(221, 217, 195), Color.FromRgb(198, 217, 240), Color.FromRgb(219, 229, 241), Color.FromRgb(242, 220, 219), Color.FromRgb(235, 241, 221), Color.FromRgb(229, 224, 236), _
            Color.FromRgb(219, 238, 243), Color.FromRgb(253, 234, 218), Color.FromRgb(216, 216, 216), Color.FromRgb(89, 89, 89), Color.FromRgb(196, 189, 151), Color.FromRgb(141, 179, 226), _
            Color.FromRgb(184, 204, 228), Color.FromRgb(229, 185, 183), Color.FromRgb(215, 227, 188), Color.FromRgb(204, 193, 217), Color.FromRgb(183, 221, 232), Color.FromRgb(251, 213, 181), _
            Color.FromRgb(191, 191, 191), Color.FromRgb(63, 63, 63), Color.FromRgb(147, 137, 83), Color.FromRgb(84, 141, 212), Color.FromRgb(149, 179, 215), Color.FromRgb(217, 150, 148), _
            Color.FromRgb(195, 214, 155), Color.FromRgb(178, 162, 199), Color.FromRgb(146, 205, 220), Color.FromRgb(250, 192, 143), Color.FromRgb(165, 165, 165), Color.FromRgb(38, 38, 38), _
            Color.FromRgb(73, 68, 41), Color.FromRgb(23, 54, 93), Color.FromRgb(54, 96, 146), Color.FromRgb(149, 55, 52), Color.FromRgb(118, 146, 60), Color.FromRgb(95, 73, 122), _
            Color.FromRgb(49, 133, 155), Color.FromRgb(227, 108, 9), Color.FromRgb(127, 127, 127), Color.FromRgb(12, 12, 12), Color.FromRgb(29, 27, 16), Color.FromRgb(15, 36, 62), _
            Color.FromRgb(36, 64, 97), Color.FromRgb(99, 36, 35), Color.FromRgb(79, 97, 40), Color.FromRgb(63, 49, 81), Color.FromRgb(32, 88, 103), Color.FromRgb(151, 72, 6) _
            }

    ' The list of standard colors, whose length should be equal to GalleryColumnItems
    Private Shared StandardColors As Color() = New Color() {Color.FromRgb(192, 0, 0), Color.FromRgb(255, 0, 0), Color.FromRgb(255, 192, 0), Color.FromRgb(255, 255, 0), Color.FromRgb(146, 208, 80), Color.FromRgb(0, 176, 80), _
            Color.FromRgb(0, 176, 240), Color.FromRgb(0, 112, 192), Color.FromRgb(0, 32, 96), Color.FromRgb(112, 48, 160)}

#End Region

#Region "Constructors"

    Shared Sub New()
        ' Override the default values for MinDropDownColumns and MaxDropDownColumns.
        ' This has the benefit over simply setting the propery in the constructor in that if
        ' the user decides to change this property and then later reset it through a call to
        ' ClearValue(...), it will be reset to our new default instead of the GalleryTool default.
        MinDropDownColumnsProperty.OverrideMetadata(GetType(ColorPickerGalleryTool), New FrameworkPropertyMetadata(GalleryColumnItems))
        MaxDropDownColumnsProperty.OverrideMetadata(GetType(ColorPickerGalleryTool), New FrameworkPropertyMetadata(GalleryColumnItems))
    End Sub

    Public Sub New()
        MyBase.New()
        ' We never want to display any text, so set the TextDisplayMode so that there
        ' will not be any space reserved for a text element
        Me.ItemSettings = New GalleryItemSettings()
        Me.ItemSettings.TextDisplayMode = GalleryItemTextDisplayMode.Never

        ' Create the border pen and the image rectangle here since they will be the same
        ' for all of the items
        Dim borderPen As New Pen(New SolidColorBrush(Color.FromRgb(197, 197, 197)), 1)
        Dim imageRect As New RectangleGeometry(New System.Windows.Rect(0, 0, 13, 13))

        Dim themeColorGroup As New GalleryItemGroup()
        themeColorGroup.Title = "Theme Colors"
        For i As Integer = 0 To ThemeColors.Length - 1
            Dim themeColor As Color = ThemeColors(i)

            ' Create a new GalleryItem for the color and store the color in the Tag
            Dim themeItem As New GalleryItem()
            themeItem.Key = [String].Format("Themed Color {0}", i.ToString())
            themeItem.Tag = themeColor

            ' Create the drawing that will display the color of the item
            Dim imageGeometryDrawing As New GeometryDrawing(New SolidColorBrush(themeColor), borderPen, imageRect)

            themeItem.Image = New DrawingImage(imageGeometryDrawing)

            ' Add the key of the item to the group and add the item itself to the tool's
            ' collection of items.
            themeColorGroup.ItemKeys.Add(themeItem.Key)
            Me.Items.Add(themeItem)
        Next
        Me.Groups.Add(themeColorGroup)

        Dim standardColorGroup As New GalleryItemGroup()
        standardColorGroup.Title = "Standard Colors"
        For j As Integer = 0 To StandardColors.Length - 1
            Dim standardColor As Color = StandardColors(j)

            ' Create a new GalleryItem for the color and store the color in the Tag
            Dim standardItem As New GalleryItem()
            standardItem.Key = [String].Format("Standard Color {0}", j.ToString())
            standardItem.Tag = standardColor

            ' Create the drawing that will display the color of the item
            Dim imageGeometryDrawing As New GeometryDrawing(New SolidColorBrush(standardColor), borderPen, imageRect)

            standardItem.Image = New DrawingImage(imageGeometryDrawing)

            ' Add the key of the item to the group and add the item itself to the tool's
            ' collection of items.
            standardColorGroup.ItemKeys.Add(standardItem.Key)
            Me.Items.Add(standardItem)
        Next

        Me.Groups.Add(standardColorGroup)
    End Sub
#End Region
End Class

