Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Shapes
Imports Infragistics.Windows.Ribbon
Imports Infragistics.Windows.Ribbon.Events
Imports System.IO
Imports Infragistics.Windows.Helpers

''' <summary>
''' Interaction logic for MyRibbonWindow.xaml
''' </summary>
Partial Public Class MyRibbonWindow
    Inherits XamRibbonWindow

#Region "Members"

    Private isUpdatingFontInfo As Boolean
    Private selectionValid As Boolean
    Private oldTextInfo As MemoryStream

#End Region

#Region "Constructor"

    ''' <summary>
    ''' Initializes a new instance the class.
    ''' </summary>
    Public Sub New()
        InitializeComponent()
    End Sub
#End Region

#Region "Properties"

#Region "IsDocumentOpen"

    ''' <summary>
    ''' Identifies the "IsDocumentOpen" property.
    ''' </summary>
    Public Shared ReadOnly IsDocumentOpenProperty As DependencyProperty = _
        DependencyProperty.Register("IsDocumentOpen", GetType(Boolean), _
        GetType(MyRibbonWindow), New FrameworkPropertyMetadata(KnownBoxes.TrueBox))

    ''' <summary>
    ''' Gets or sets whether there is currently a document editing surface shown.
    ''' </summary>
    Public Property IsDocumentOpen() As Boolean
        Get
            Return CBool(Me.GetValue(IsDocumentOpenProperty))
        End Get
        Set(ByVal value As Boolean)
            Me.SetValue(IsDocumentOpenProperty, value)
        End Set
    End Property
#End Region

#Region "IsSelectionValid"

    ''' <summary>
    ''' Gets whether the any text is currently selected in the associated RichTextBox
    ''' </summary>
    Public Property IsSelectionValid() As Boolean
        Get
            Return Me.selectionValid
        End Get
        Set(ByVal value As Boolean)
            If Me.selectionValid <> value Then
                Me.selectionValid = value
                Me.myRibbon.ContextualTabGroups(0).IsVisible = value
            End If
        End Set
    End Property
#End Region

#End Region

#Region "Event Handlers"

#Region "OnClipArtButtonClick"

    Private Sub OnClipArtButtonClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Dim tool As ButtonTool = TryCast(e.Source, ButtonTool)
        If tool IsNot Nothing Then
            Dim image As New Image()
            image.Width = tool.LargeImage.Width
            image.Source = tool.LargeImage

            Dim p As New Paragraph(New InlineUIContainer(image, Me.richTextBox.Selection.Start))
            Me.richTextBox.Document.Blocks.Add(p)
        End If
    End Sub
#End Region

#Region "OnCloseButtonClick"

    Private Sub OnCloseButtonClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Me.SaveContent(False)
        Me.richTextBox.Document.Blocks.Clear()
        Me.richTextBox.Visibility = Visibility.Hidden
    End Sub
#End Region

#Region "OnColorPicked"

    Private Sub OnColorPicked(ByVal sender As Object, ByVal e As GalleryItemEventArgs)
        If e.Item Is Nothing OrElse e.Item.Tag Is Nothing Then
            Return
        End If

        Dim newColor As Color = DirectCast(e.Item.Tag, Color)
        Dim newBrush As New SolidColorBrush(newColor)
        If e.GalleryTool.Id = "globalColorPicker" Then
            Me.richTextBox.SetValue(richTextBox.ForegroundProperty, newBrush)
        ElseIf e.GalleryTool.Id = "selectionColorPicker" Then
            Me.richTextBox.Selection.ApplyPropertyValue(richTextBox.ForegroundProperty, newBrush)
        End If
    End Sub
#End Region

#Region "OnFontInfoChanged"

    Private Sub OnFontInfoChanged(ByVal sender As Object, ByVal e As RoutedPropertyChangedEventArgs(Of Object))
        If Me.isUpdatingFontInfo Then
            Return
        End If

        If e.Source Is Me.fontFamilyCombo_Selection Then
            Dim fontFamily As FontFamily = DirectCast(Me.fontFamilyCombo_Selection.SelectedItem, FontFamily)
            Me.richTextBox.Selection.ApplyPropertyValue(TextElement.FontFamilyProperty, fontFamily)
        ElseIf e.Source Is Me.fontSizeCombo_Selection AndAlso Me.fontSizeCombo_Selection.Value IsNot Nothing Then
            Dim fontSize As Double = DirectCast(Me.fontSizeCombo_Selection.SelectedItem, Double)
            Me.richTextBox.Selection.ApplyPropertyValue(TextElement.FontSizeProperty, fontSize)
        End If
    End Sub
#End Region

#Region "OnGalleryItemActivated"

    ' We want to set either the selected text or the entire document's style to whatever
    ' the associated style of the gallery item is, which is the Live Preview feature in Office 2007
    Private Sub OnGalleryItemActivated(ByVal sender As Object, ByVal e As GalleryItemEventArgs)
        Dim fontFamily As FontFamily
        Dim fontSize As Double
        Dim foreBrush As SolidColorBrush
        Dim fontStyle As String

        If e.Item IsNot Nothing AndAlso e.Item.Tag IsNot Nothing Then
            Me.ParseGalleryItemTag(e.Item.Tag.ToString(), fontFamily, fontSize, foreBrush, fontStyle)
            Me.ApplyStyleToEditor(fontFamily, fontSize, foreBrush, fontStyle)
        ElseIf Me.oldTextInfo IsNot Nothing Then
            If Me.IsSelectionValid Then
                Me.richTextBox.Selection.Load(Me.oldTextInfo, DataFormats.XamlPackage)
            Else
                Dim range As TextRange = New TextRange(Me.richTextBox.Document.ContentStart, Me.richTextBox.Document.ContentEnd)
                range.Load(Me.oldTextInfo, DataFormats.XamlPackage)
            End If
        End If

    End Sub
#End Region

#Region "OnGalleryItemSelected"

    Private Sub OnGalleryItemSelected(ByVal sender As Object, ByVal e As GalleryItemEventArgs)
        Dim fontFamily As FontFamily
        Dim fontSize As Double
        Dim foreBrush As SolidColorBrush
        Dim fontStyle As String

        Me.ParseGalleryItemTag(e.Item.Tag.ToString(), fontFamily, fontSize, foreBrush, fontStyle)
        Me.ApplyStyleToEditor(fontFamily, fontSize, foreBrush, fontStyle)
    End Sub
#End Region

#Region "OnGalleryMenuOpened"

    Private Sub OnGalleryMenuOpened(ByVal sender As Object, ByVal e As RoutedEventArgs)
        If Me.oldTextInfo IsNot Nothing Then
            Me.oldTextInfo.Close()
        End If

        Me.oldTextInfo = New MemoryStream()
        If (Me.IsSelectionValid) Then
            Me.richTextBox.Selection.Save(Me.oldTextInfo, DataFormats.XamlPackage)
        Else
            Dim range As TextRange = New TextRange(Me.richTextBox.Document.ContentStart, Me.richTextBox.Document.ContentEnd)
            range.Save(Me.oldTextInfo, DataFormats.XamlPackage)
        End If
    End Sub
#End Region

#Region "OnNewButtonClick"

    Private Sub OnNewButtonClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Me.CreateNewDocument()
    End Sub
#End Region

#Region "OnOpenButtonClick"

    Private Sub OnOpenButtonClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Me.SaveContent(False)

        Dim dlg As New Microsoft.Win32.OpenFileDialog()
        dlg.Filter = "Xaml Package File (*.xpkg)|*.xpkg;"
        If dlg.ShowDialog(Me).Value = True Then
            Dim stream As FileStream = Nothing
            Try
                stream = New FileStream(dlg.FileName, FileMode.Open)
                Dim range As New TextRange(Me.richTextBox.Document.ContentStart, Me.richTextBox.Document.ContentEnd)
                range.Load(stream, DataFormats.XamlPackage)
                Me.richTextBox.Visibility = Windows.Visibility.Visible
            Catch ex As IOException
                MessageBox.Show("There was an error opening the specified file: " & Chr(10) & "" & Chr(10) & "" + ex.Message)
            Finally
                If stream IsNot Nothing Then
                    stream.Close()
                End If
            End Try
        End If
    End Sub
#End Region

#Region "OnRecentItemClick"

    Private Sub OnRecentItemClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Dim item As ButtonTool = TryCast(e.Source, ButtonTool)
        If item IsNot Nothing Then
            Me.CreateNewDocument()
            Me.richTextBox.Document.Blocks.Clear()
            Dim r As New Run(item.Content.ToString())
            Me.richTextBox.Document.Blocks.Add(New Paragraph(r))
        End If
    End Sub
#End Region

#Region "OnRichTextBoxSelectionChanged"

    Private Sub OnRichTextBoxSelectionChanged(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Dim selection As TextSelection = Me.richTextBox.Selection
        Me.boldTool.IsChecked = selection.GetPropertyValue(FontWeightProperty).ToString() = "Bold"
        Me.italicTool.IsChecked = selection.GetPropertyValue(FontStyleProperty).ToString() = "Italic"
        Me.underlineTool.IsChecked = selection.GetPropertyValue(Inline.TextDecorationsProperty) Is TextDecorations.Underline

        ' We only want to allow selection-specific actions when there is actually something selected
        Me.IsSelectionValid = selection.Start IsNot selection.End

        Me.SynchronizeSelectionCombos()
    End Sub
#End Region

#Region "OnSaveButtonClick"

    Private Sub OnSaveButtonClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Me.SaveContent(True)
    End Sub
#End Region

#Region "OnThemeSelected"

    Private Sub OnThemeSelected(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Dim tool As RadioButtonTool = TryCast(e.Source, RadioButtonTool)
        If tool IsNot Nothing Then
            Me.myRibbon.Theme = tool.Tag.ToString()
        End If
    End Sub
#End Region

#Region "OnWindowLoad"

    Private Sub OnWindowLoad(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Me.richTextBox.Focus()
    End Sub
#End Region

#End Region

#Region "Private Methods"

#Region "ApplyStyleToEditor"

    Private Sub ApplyStyleToEditor(ByVal fontFamily As FontFamily, ByVal fontSize As Double, ByVal foreBrush As SolidColorBrush, ByVal fontStyle As String)
        ' Check to see whether we should apply the new style only to the selected text
        If Me.IsSelectionValid Then
            Dim start As TextPointer = Me.richTextBox.Selection.Start
            Dim [end] As TextPointer = Me.richTextBox.Selection.[End]

            Dim range As New TextRange(start, [end])
            range.ApplyPropertyValue(richTextBox.FontFamilyProperty, fontFamily)
            range.ApplyPropertyValue(richTextBox.FontSizeProperty, fontSize)
            range.ApplyPropertyValue(richTextBox.ForegroundProperty, foreBrush)

            If fontStyle = "Bold" Then
                range.ApplyPropertyValue(richTextBox.FontWeightProperty, FontWeights.Bold)
            Else
                range.ApplyPropertyValue(richTextBox.FontWeightProperty, richTextBox.FontWeightProperty.DefaultMetadata.DefaultValue)
            End If

            If fontStyle = "Italic" Then
                range.ApplyPropertyValue(richTextBox.FontStyleProperty, FontStyles.Italic)
            Else
                range.ApplyPropertyValue(richTextBox.FontStyleProperty, richTextBox.FontStyleProperty.DefaultMetadata.DefaultValue)
            End If
        Else
            Dim range As TextRange = New TextRange(Me.richTextBox.Document.ContentStart, Me.richTextBox.Document.ContentEnd)

            If fontFamily IsNot Nothing Then
                range.ApplyPropertyValue(richTextBox.FontFamilyProperty, fontFamily)
            End If

            If fontSize > 0 Then
                range.ApplyPropertyValue(richTextBox.FontSizeProperty, fontSize)
            End If

            If foreBrush IsNot Nothing Then
                range.ApplyPropertyValue(richTextBox.ForegroundProperty, foreBrush)
            End If

            If fontStyle = "Bold" Then
                range.ApplyPropertyValue(richTextBox.FontWeightProperty, FontWeights.Bold)
            Else
                range.ApplyPropertyValue(richTextBox.FontWeightProperty, richTextBox.FontWeightProperty.DefaultMetadata.DefaultValue)
            End If

            If fontStyle = "Italic" Then
                range.ApplyPropertyValue(richTextBox.FontStyleProperty, FontStyles.Italic)
            Else
                range.ApplyPropertyValue(richTextBox.FontStyleProperty, richTextBox.FontStyleProperty.DefaultMetadata.DefaultValue)
            End If

        End If
    End Sub
#End Region

#Region "CreateNewDocument"

    Private Sub CreateNewDocument()
        Me.SaveContent(False)
        Me.richTextBox.Document.Blocks.Clear()
        Me.richTextBox.Visibility = Visibility.Visible
    End Sub
#End Region

#Region "ParseGalleryItemTag"

    Private Sub ParseGalleryItemTag(ByVal tag As String, ByRef fontFamily As FontFamily, ByRef fontSize As Double, ByRef foreBrush As SolidColorBrush, ByRef fontStyle As String)
        Dim styleInfo As String() = tag.Split(";"c)

        ' Parse the font family
        fontFamily = New FontFamily(Nothing, styleInfo(0))
        fontSize = Double.Parse(styleInfo(1))

        ' Parse the fore-color
        Dim rgb As String() = styleInfo(2).Split(","c)
        Dim fontColor As Color = Color.FromRgb(Byte.Parse(rgb(0)), Byte.Parse(rgb(1)), Byte.Parse(rgb(2)))
        foreBrush = New SolidColorBrush(fontColor)

        fontStyle = styleInfo(3)
    End Sub
#End Region

#Region "SaveContent"

    Private Sub SaveContent(ByVal isUserTriggered As Boolean)
        Dim document As FlowDocument = Me.richTextBox.Document
        Dim range As New TextRange(document.ContentStart, document.ContentEnd)

        Dim isDocumentEmpty As Boolean = range.Text = "" & Chr(13) & "" & Chr(10) & "" OrElse range.Text.Length = 0

        ' Don't bother showing the dialog if we have nothing to save and the user
        ' didn't specifically request the save
        If isDocumentEmpty AndAlso isUserTriggered = False Then
            Return
        End If

        ' Prompt the user on whether they want to save the content if this function were triggered
        ' by another action (i.e. New, Close)
        If isUserTriggered = False Then
            If MessageBoxResult.No = MessageBox.Show(Me, "There are unsaved changes. Would you like to save?", "Save File", MessageBoxButton.YesNo) Then
                Return
            End If
        End If

        Dim dlg As New Microsoft.Win32.SaveFileDialog()
        dlg.Filter = "Xaml Package File (*.xpkg)|*.xpkg;"
        If dlg.ShowDialog(Me).Value = True Then
            Dim stream As FileStream = Nothing
            Try
                stream = New FileStream(dlg.FileName, FileMode.Create)
                range.Save(stream, DataFormats.XamlPackage)
            Catch e As IOException
                MessageBox.Show("Could not save file: " & Chr(10) & "" & Chr(10) & "" + e.Message)
            Finally
                If stream IsNot Nothing Then
                    stream.Close()
                End If
            End Try
        End If
    End Sub
#End Region

#Region "SynchronizeSelectionCombos"

    Private Sub SynchronizeSelectionCombos()
        Me.isUpdatingFontInfo = True

        Me.fontFamilyCombo_Selection.SelectedItem = Me.richTextBox.Selection.GetPropertyValue(richTextBox.FontFamilyProperty)

        Me.fontSizeCombo_Selection.SelectedItem = Me.richTextBox.Selection.GetPropertyValue(richTextBox.FontSizeProperty)

        Me.isUpdatingFontInfo = False
    End Sub
#End Region

#End Region

End Class
